	.length 78
	.width  132
;************************************************************************
;									*
;				S208AT2.ASM				*
;									*
;************************************************************************
; C40 test program which only requires C40 internal RAM BLK 0 and 1.
; It should be loaded using the Emulator over the JTAG interface.
; I/O is restricted to the Emulator display/edit facilities.
;
; History:
; 09-Jun-96 SEC Memory tests:
;		1. Self address
;		2. All 1's
;		3. All 0's
;		4. Alternate 0 and 1 words
;		5. Rotating single bit.
;

;************************************************************************
;	EQUATES								*
;************************************************************************

; A flash_bit of 0001 gives a 1Hz flash with 64KW mem test:
flash_bit	.equ	0001h		; bit in pass count to flash

ramblk0_st	.equ	002FF800h	; Start of internal sram
start		.equ	002FF900h	; Code start address
stack		.equ	002FFE00h	; Stack start address, grows up
ramblk1_end	.equ	002FFFFFh	; End of internal sram

; SMT208 with 64K words (256KB) local sram:
; 1 pass takes around 0.5 seconds on a 40MHz 208 for all 64KW:
;ram_start	.equ	00300000h	; Ram start for test
;ram_end		.equ	0030FFFFh

; SMT208 with 2M words (8MB) global dram:
; 1 pass takes 25 seconds on a 40MHz 208 for all 8MB:
;ram_start	.equ	80000000h	; Ram start for test
;ram_end		.equ	801FFFFFh	; Ram end for test

; SMT208A with 256K words (1MB) local sram:
ram_start	.equ	00301000h	; Ram start for test
ram_end		.equ	0033FFFFh	; Ram end for test

; SMT208 with 256K words (1MB) global sram:
;ram_start	.equ	80000000h	; Ram start for test
;ram_end		.equ	8003FFFFh	; Ram end for test

; SMT208 with 1M words (4MB) global edram:
;ram_start	.equ	80040000h	; Ram start for test
;ram_end		.equ	8013FFFFh	; Ram end for test



link0		.equ	40000000h	; Link 0 c011
link1		.equ	40000004h	; Link 1 c011
link2		.equ	40000008h	; Link 2 c011
link3		.equ	4000000Ch	; Link 3 c011

led		.equ	40000010h	; Led0

link_din	.equ	0		; c011 data in
link_dout	.equ	1		; c011 data out
link_statin	.equ	2		; c011 data in status
link_statout	.equ	3		; c011 data out status

link_test	.equ	link0		; link to loopback


;************************************************************************
;	CODE								*
;************************************************************************

	.global	_main

; The program leaves space for the ID ROM data at ramblk0_st:

	.text

; Disable interrupts, repeat off, cache off:
_main:	ldi	0,st			; clear status reg
	ldp	@stack,dp		; get stack page
	ldhi	stack/10000h,sp		; set sp msb
	or	stack&0ffffh,sp		; set sp lsb


	ldi	0,r4			; pass count

; Test 1. Self address test
tst01:	ldhi	ram_start/10000h,r0	; set seed data
	or	ram_start&0ffffh,r0	;
	ldi	1,r1			; set data increment
	ldi	0,r2			; set data left rotate
	call	fill			; fill ram
	call	read			; read back
	call	read			; and again

; Test 2. All 1's
	ldhi	0ffffh,r0		; set seed data
	or	0ffffh,r0		;
	ldi	0,r1			; set data increment
	ldi	0,r2			; set data left rotate
	call	fill			; fill ram
	call	read			; read back
	call	read			; and again

; Test 3. All 0's
	ldhi	0h,r0			; set seed data
	or	0h,r0			;
	ldi	0,r1			; set data increment
	ldi	0,r2			; set data left rotate
	call	fill			; fill ram
	call	read			; read back
	call	read			; and again

; Test 4. Alternate 0 and 1 words
	ldhi	0h,r0			; set seed data
	or	0h,r0			;
	ldi	0h,r1			; set data increment
	ldhi	0ffffh,r2		; set data XOR
	or	0ffffh,r2		;
	call	fill			; fill ram
	call	read			; read back
	call	read			; and again

; Test 5. Rotating single bit.
	ldhi	0h,r0			; set seed data
	or	1h,r0			;
	ldi	0,r1			; set data increment
	ldi	1,r2			; set data left rotate
	call	fill			; fill ram
	call	read			; read back
	call	read			; and again

; Inc pass count and flash LED:
	addi	1,r4			; inc pass count
	call	flash			; flash led
	br	tst01			; loop


; Flash LED using pass count in r4:
flash:  ldhi	led/10000h,ar7		; address led
        or	led&0FFFFh,ar7		;
	ldi	0,r5			; guess led is off
	tstb	flash_bit,r4		; led on or off ?
	beq	flash1			; off
	ldi	1,r5			; led on
flash1: sti	r5,*ar7			; set LED bit
	rets

; on error halt:
error:	br	error

; Fill ram using data as follows:
; r0 = Seed data
; r1 = Increment
; r2 = Left shift 1 if bit 0 set, XOR is bit 1 set
fill:	push	r0			; save seed
	ldhi	ram_start/10000h,ar0	; set start address
	or	ram_start&0ffffh,ar0	;
	ldhi	ram_end/10000h,ar1	; set end address
	or	ram_end&0ffffh,ar1	;
fill1:	sti	r0,*ar0++		; write data
	addi	r1,r0			; add increment
; test for XOR:
	tstb	2,r2			; XOR ?
	beq	fill2			; no
	xor	r2,r0			; XOR
	br	fill3
; test for rotate:
fill2:	tstb	1,r2			; rotate ?
	beq	fill3			; no
	rol	r0			; rotate left
fill3:
	cmpi	ar0,ar1			; end ?
	bne	fill1			; no
	pop	r0			; fetch seed
	rets


; Read from ram using data as follows:
; r0 = Seed data
; r1 = Increment
; r2 = Left shift 1 if bit 0 set, XOR is bit 1 set
read:	push	r0			; save seed
	ldhi	ram_start/10000h,ar0	; set start address
	or	ram_start&0ffffh,ar0	;
	ldhi	ram_end/10000h,ar1	; set end address
	or	ram_end&0ffffh,ar1	;
read1:	ldi	*ar0++,r3		; read data
	cmpi	r0,r3			; correct ?
	bne	error			; no halt
	addi	r1,r0			; add increment
; test for XOR:
	tstb	2,r2			; XOR ?
	beq	read2			; no
	xor	r2,r0			; XOR
	br	read3			;
; test for rotate:
read2:	tstb	1,r2			; rotate ?
	beq	read3			; no
	rol	r0			; rotate left
read3:
	cmpi	ar0,ar1			; end ?
	bne	read1			; no
	pop	r0			; fetch seed
	rets

	.end

