	.length 78
	.width  132
;************************************************************************
;									*
;				S208AT6.ASM				*
;									*
;************************************************************************
; C40 test program which only requires C40 internal RAM BLK 0 and 1.
; It should be loaded using the Emulator over the JTAG interface.
; I/O is restricted to the Emulator display/edit facilities.
;
; History:
; 09-Jun-96 SEC High speed I/O port test
;

;************************************************************************
;	EQUATES								*
;************************************************************************

; A flash_bit of 0001 gives a 1Hz flash with 64KW mem test:
flash_bit	.equ	0001h		; bit in pass count to flash

ramblk0_st	.equ	002FF800h	; Start of internal sram
start		.equ	002FF900h	; Code start address
stack		.equ	002FFE00h	; Stack start address, grows up
ramblk1_end	.equ	002FFFFFh	; End of internal sram

; SMT208 with 64K words (256KB) local sram:
; 1 pass takes around 0.5 seconds on a 40MHz 208 for all 64KW:
;ram_start	.equ	00300000h	; Ram start for test
;ram_end		.equ	0030FFFFh

; SMT208 with 2M words (8MB) global dram:
; 1 pass takes 25 seconds on a 40MHz 208 for all 8MB:
;ram_start	.equ	80000000h	; Ram start for test
;ram_end		.equ	801FFFFFh	; Ram end for test

; SMT208A with 256K words (1MB) local sram:
;ram_start	.equ	00300000h	; Ram start for test
;ram_end		.equ	0033FFFFh	; Ram end for test

; SMT208A with 256K words (1MB) global sram:
;ram_start	.equ	80000000h	; Ram start for test
;ram_end		.equ	8001FFFFh	; Ram end for test

; SMT208A with 1M words (4MB) global edram:
ram_start	.equ	80020000h	; Ram start for test
ram_end		.equ	8011FFFFh	; Ram end for test

iop		.equ	40000010h	; IO port to test

link0		.equ	40000000h	; Link 0 c011
link1		.equ	40000004h	; Link 1 c011
link2		.equ	40000008h	; Link 2 c011
link3		.equ	4000000Ch	; Link 3 c011

led		.equ	40000010h	; Led0

link_din	.equ	0		; c011 data in
link_dout	.equ	1		; c011 data out
link_statin	.equ	2		; c011 data in status
link_statout	.equ	3		; c011 data out status

link_test	.equ	link0		; link to loopback


;************************************************************************
;	CODE								*
;************************************************************************

	.global	_main

; The program leaves space for the ID ROM data at ramblk0_st:

	.text

; Disable interrupts, repeat off, cache off:
_main:	ldi	0,st			; clear status reg
	ldp	@stack,dp		; get stack page
	ldhi	stack/10000h,sp		; set sp msb
	or	stack&0ffffh,sp		; set sp lsb

	ldhi	iop/10000h,ar0
	or	iop&0ffffh,ar0

	ldi	0,r2
	ldi	-1,r3

loop0:	sti	r2,*ar0
	ldi	*ar0,r0
	sti	r3,*ar0
	ldi	*ar0,r0
	br	loop0

	.end

