
/*--------------------------------------------------*/
/*                                                  */
/* TRAMIO.H                                         */
/*                                                  */
/* This header file contains the functions required */
/* to communicate with the Tram monitor.            */
/*                                                  */
/*--------------------------------------------------*/


#define	byte		unsigned char

/*------------------------*/
/* Misc Trans definitions */
/*------------------------*/

#define LINK_TIMEOUT	5000
#define ON_LINE		0xf4
#define SSRESETADDR	0x00000000


/*--------------------------------------*/
/* Link Control Bit Field Definitions	*/
/*--------------------------------------*/

#define	MIOCMD		0x40	/* Memory Command (1 = Mem I/O = 0)    */
#define	WRCMD		0x80	/* Write  Command (1 = Write 0 = Read) */
#define	SUBCMD		0x01	/* Memory CALL Command                 */



/*-------------------------*/
/* Reset the TRAM function */
/*-------------------------*/
void ResetTram(Channel *LinkIn, Channel *LinkOut)
{
	int	i;

	char	*ResetPtr=(char *)SSRESETADDR;


	/*--------------------*/
	/* Reset the channels */
	/*--------------------*/
	
	ChanReset(LinkIn);
	ChanInit(LinkIn);
	
	ChanReset(LinkOut);
	ChanInit(LinkOut);			

	
	/*------------------------------------------*/
	/* Set the reset line low for a short time. */
	/*------------------------------------------*/
	*ResetPtr = 0xff;
	for(i=0; i<5000; i++);
	*ResetPtr = 0;			
}


/*-----------------------------*/
/* Write a Byte to an I/O port */
/*-----------------------------*/
int WriteIOByte(Channel *LinkOut, byte port, byte data)
{
	byte	WriteData[3];
	
	WriteData[0] = WRCMD;
	WriteData[1] = port;
	WriteData[2] = data;
	
	return	ChanOutTimeFail(LinkOut, WriteData, 3,	
		                ProcTimePlus(ProcTime(), LINK_TIMEOUT));
}


/*------------------------------*/
/* Read a Byte from an I/O port */
/*------------------------------*/
int ReadIOByte(Channel *LinkIn, Channel *LinkOut, byte IOPort, byte *ReadPtr)
{
	byte	WriteData[2];
	int	ReadState;

	WriteData[0] = 0;
	WriteData[1] = IOPort;

	if(ChanOutTimeFail(LinkOut, &WriteData, 2,	
		                ProcTimePlus(ProcTime(), LINK_TIMEOUT)) == 1)
		return 1;

	ReadState = ChanInTimeFail(LinkIn, ReadPtr, 1,	
		                ProcTimePlus(ProcTime(), LINK_TIMEOUT));


	return ReadState;
}



/*-----------------------------------------*/
/* Write a Data Block to a Memory Location */
/*-----------------------------------------*/
int WriteTramMem(Channel *LinkOut, int MemAddr, byte DataLength, byte *DataPtr)
{
	byte	WriteSetup[4];

	
	WriteSetup[0] = MIOCMD | WRCMD;
	WriteSetup[1] = (byte) (MemAddr & 0xff);
	WriteSetup[2] = (byte) (MemAddr >> 8);
	WriteSetup[3] = DataLength;	

	if(ChanOutTimeFail(LinkOut, WriteSetup, 4,	
		                ProcTimePlus(ProcTime(), LINK_TIMEOUT)) == 1)
		return 1;

	return	ChanOutTimeFail(LinkOut, DataPtr, DataLength,	
		                ProcTimePlus(ProcTime(), LINK_TIMEOUT));
}


/*------------------------------------------*/
/* Read a Data Block from a Memory Location */
/*------------------------------------------*/
int ReadTramMem(Channel *LinkIn, Channel *LinkOut,
			int MemAddr, byte DataLength, byte *DataPtr)
{
	
	char	ReadSetup[4];
	
	ReadSetup[0] = MIOCMD;
	ReadSetup[1] = (byte) (MemAddr & 0xff);	
	ReadSetup[2] = (byte) (MemAddr >> 8);
	ReadSetup[3] = DataLength;


	if(ChanOutTimeFail(LinkOut, ReadSetup, 4,	
		                ProcTimePlus(ProcTime(), LINK_TIMEOUT)) == 1)
		return 1;
		

	return ChanInTimeFail(LinkIn, DataPtr, DataLength,	
		                ProcTimePlus(ProcTime(), LINK_TIMEOUT));
	
}


/*-----------------*/
/* Call a function */
/*-----------------*/
int CallFunction(Channel *LinkOut)
{
	char	CallCmd;
	
	CallCmd = MIOCMD | SUBCMD;

	return	ChanOutTimeFail(LinkOut, &CallCmd, 1,	
		                ProcTimePlus(ProcTime(), LINK_TIMEOUT));
}

